@@ -37,7 +37,14 @@ except ImportError:
     ZIP_COMPRESSION_MODE = zipfile.ZIP_STORED
 
 
-class BinaryStdout(object):
+class NonTranslatedStdout(object):
+    """ This context manager sets the line-end translation mode for stdout.
+
+    It is deliberately set to binary mode so that `\r` does not get added to
+    the line ending. This can be useful when printing commands where a
+    windows style line ending would casuse errors.
+    """
+
     def __enter__(self):
         if sys.platform == "win32":
             import msvcrt
@@ -48,7 +55,7 @@ class BinaryStdout(object):
     def __exit__(self, type, value, traceback):
         if sys.platform == "win32":
             import msvcrt
-            msvcrt.setmode(sys.stdout.fileno(), self.previous_mode)                
+            msvcrt.setmode(sys.stdout.fileno(), self.previous_mode)
 
 
 if six.PY3:
@@ -59,6 +66,8 @@ if six.PY3:
 
     raw_input = input
 
+    binary_stdin = sys.stdin.buffer
+
     def get_stdout_text_writer():
         return sys.stdout
 
@@ -77,6 +86,20 @@ if six.PY3:
             encoding = locale.getpreferredencoding()
         return open(filename, mode, encoding=encoding)
 
+    def bytes_print(statement, stdout=None):
+        """
+        This function is used to write raw bytes to stdout.
+        """
+        if stdout is None:
+            stdout = sys.stdout
+
+        if getattr(stdout, 'buffer', None):
+            stdout.buffer.write(statement)
+        else:
+            # If it is not possible to write to the standard out buffer.
+            # The next best option is to decode and write to standard out.
+            stdout.write(statement.decode('utf-8'))
+
 else:
     import codecs
     import locale
@@ -87,6 +110,8 @@ else:
 
     raw_input = raw_input
 
+    binary_stdin = sys.stdin
+
     def get_stdout_text_writer():
         # In python3, all the sys.stdout/sys.stderr streams are in text
         # mode.  This means they expect unicode, and will encode the
@@ -106,6 +131,12 @@ else:
             encoding = locale.getpreferredencoding()
         return io.open(filename, mode, encoding=encoding)
 
+    def bytes_print(statement, stdout=None):
+        if stdout is None:
+            stdout = sys.stdout
+
+        stdout.write(statement)
+
 
 def compat_input(prompt):
     """
